@@ -17,26 +17,24 @@ from cftool.array import to_torch
 from cftool.types import tensor_dict_type
 
 from ..types import losses_type
-from ..schema import MultiLoss
 from ..schema import ILoss
 from ..schema import TrainerState
-from ..register import register_loss_module
 from ..constants import LOSS_KEY
 from ..constants import INPUT_KEY
 from ..constants import LABEL_KEY
 from ..constants import PREDICTIONS_KEY
 
 
-@register_loss_module("iou")
-class IOULoss(nn.Module):
+@ILoss.register("iou")
+class IOULoss(ILoss):
     def forward(self, logits: Tensor, labels: Tensor) -> Tensor:
         return 1.0 - iou(logits, labels)
 
 
-@register_loss_module("bce")
-class BCELoss(nn.Module):
-    def __init__(self) -> None:
-        super().__init__()
+@ILoss.register("bce")
+class BCELoss(ILoss):
+    def __init__(self, reduction: str = "mean"):
+        super().__init__(reduction)
         self.bce = nn.BCEWithLogitsLoss(reduction="none")
 
     def forward(self, predictions: Tensor, labels: Tensor) -> Tensor:
@@ -44,47 +42,56 @@ class BCELoss(nn.Module):
         return losses.mean(tuple(range(1, len(losses.shape))))
 
 
-@register_loss_module("mae")
-class MAELoss(nn.Module):
+@ILoss.register("mae")
+class MAELoss(ILoss):
     def forward(self, predictions: Tensor, labels: Tensor) -> Tensor:
         return F.l1_loss(predictions, labels, reduction="none")
 
 
-@register_loss_module("sigmoid_mae")
-class SigmoidMAELoss(nn.Module):
+@ILoss.register("sigmoid_mae")
+class SigmoidMAELoss(ILoss):
     def forward(self, predictions: Tensor, labels: Tensor) -> Tensor:
         losses = F.l1_loss(torch.sigmoid(predictions), labels, reduction="none")
         return losses.mean((1, 2, 3))
 
 
-@register_loss_module("mse")
-class MSELoss(nn.Module):
+@ILoss.register("mse")
+class MSELoss(ILoss):
     def forward(self, predictions: Tensor, labels: Tensor) -> Tensor:
         return F.mse_loss(predictions, labels, reduction="none")
 
 
-@register_loss_module("recon")
-class ReconstructionLoss(nn.Module):
+@ILoss.register("recon")
+class ReconstructionLoss(ILoss):
     base_loss: ILoss
 
-    def __init__(self, base_loss_name: str, **kwargs: Any):
-        super().__init__()
-        self.base_loss = ILoss.make(base_loss_name, kwargs)
+    def __init__(
+        self, reduction: str = "mean", *, base_loss: str = "mae", **kwargs: Any
+    ):
+        super().__init__(reduction)
+        self.base_loss = ILoss.make(base_loss, kwargs)
+
+    def get_forward_args(
+        self,
+        forward_results: tensor_dict_type,
+        batch: tensor_dict_type,
+        state: Optional[TrainerState] = None,
+    ) -> Tuple[Any, ...]:
+        return forward_results, batch, state
 
     def forward(
         self,
         forward_results: tensor_dict_type,
         batch: tensor_dict_type,
         state: Optional[TrainerState] = None,
-        **kwargs: Any,
     ) -> losses_type:
         batch = shallow_copy_dict(batch)
         batch[LABEL_KEY] = batch[INPUT_KEY]
-        return self.base_loss._core(forward_results, batch, state, **kwargs)
+        return self.base_loss.run(forward_results, batch, state)
 
 
-@register_loss_module("quantile")
-class QuantileLoss(nn.Module):
+@ILoss.register("quantile")
+class QuantileLoss(ILoss):
     def __init__(self, q: Union[float, List, Tuple, np.ndarray]):
         super().__init__()
         if isinstance(q, float):
@@ -101,14 +108,14 @@ class QuantileLoss(nn.Module):
         return quantile_losses.mean(1, keepdim=True)
 
 
-@register_loss_module("corr")
-class CorrelationLoss(nn.Module):
+@ILoss.register("corr")
+class CorrelationLoss(ILoss):
     def forward(self, predictions: Tensor, labels: Tensor) -> Tensor:
         return -corr(predictions, labels, get_diagonal=True)
 
 
-@register_loss_module("cross_entropy")
-class CrossEntropyLoss(nn.Module):
+@ILoss.register("cross_entropy")
+class CrossEntropyLoss(ILoss):
     @staticmethod
     def _get_stat(predictions: Tensor, labels: Tensor) -> Tuple[Tensor, Tensor]:
         log_prob_mat = F.log_softmax(predictions, dim=1)
@@ -119,8 +126,8 @@ class CrossEntropyLoss(nn.Module):
         return self._get_stat(predictions, labels)[1]
 
 
-@register_loss_module("label_smooth_cross_entropy")
-class LabelSmoothCrossEntropyLoss(nn.Module):
+@ILoss.register("label_smooth_cross_entropy")
+class LabelSmoothCrossEntropyLoss(ILoss):
     def __init__(self, eps: float = 0.1):
         super().__init__()
         self._eps = eps
@@ -132,8 +139,8 @@ class LabelSmoothCrossEntropyLoss(nn.Module):
         return (1.0 - self._eps) * nll_losses + eps * smooth_losses
 
 
-@register_loss_module("focal")
-class FocalLoss(nn.Module):
+@ILoss.register("focal")
+class FocalLoss(ILoss):
     def __init__(
         self,
         *,
@@ -170,53 +177,6 @@ class FocalLoss(nn.Module):
         return loss.view_as(labels)
 
 
-@MultiLoss.record_prefix()
-class MultiTaskLoss(MultiLoss):
-    prefix = "multi_task"
-
-    def _core(
-        self,
-        forward_results: tensor_dict_type,
-        batch: tensor_dict_type,
-        state: Optional[TrainerState] = None,
-        **kwargs: Any,
-    ) -> losses_type:
-        losses: losses_type = {}
-        for loss_ins in self.base_losses:
-            self._inject(
-                loss_ins.__identifier__,
-                loss_ins._core(forward_results, batch, state, **kwargs),
-                losses,
-            )
-        losses[LOSS_KEY] = sum(losses.values())
-        return losses
-
-
-@MultiLoss.record_prefix()
-class MultiStageLoss(MultiLoss):
-    prefix = "multi_stage"
-
-    def _core(
-        self,
-        forward_results: tensor_dict_type,
-        batch: tensor_dict_type,
-        state: Optional[TrainerState] = None,
-        **kwargs: Any,
-    ) -> losses_type:
-        predictions = forward_results[PREDICTIONS_KEY]
-        losses: losses_type = {}
-        for i, pred in enumerate(predictions):
-            forward_results[PREDICTIONS_KEY] = pred
-            for loss_ins in self.base_losses:
-                self._inject(
-                    f"{loss_ins.__identifier__}{i}",
-                    loss_ins._core(forward_results, batch, state, **kwargs),
-                    losses,
-                )
-        losses[LOSS_KEY] = sum(losses.values())
-        return losses
-
-
 __all__ = [
     "IOULoss",
     "MAELoss",
@@ -227,5 +187,4 @@ __all__ = [
     "CrossEntropyLoss",
     "LabelSmoothCrossEntropyLoss",
     "FocalLoss",
-    "MultiStageLoss",
 ]
