@@ -207,6 +207,7 @@ class DDPM(ModelWithCustomSteps, GaussianGeneratorMixin):
         use_linear_in_transformer: bool = False,
         use_checkpoint: bool = False,
         attn_split_chunk: Optional[int] = None,
+        tome_info: Optional[Dict[str, Any]] = None,
         # ControlNet
         only_mid_control: bool = False,
         # diffusion
@@ -262,6 +263,7 @@ class DDPM(ModelWithCustomSteps, GaussianGeneratorMixin):
             use_linear_in_transformer=use_linear_in_transformer,
             use_checkpoint=use_checkpoint,
             attn_split_chunk=attn_split_chunk,
+            tome_info=tome_info,
         )
         self.unet = UNetDiffuser(out_channels=out_channels, **self.unet_kw)  # type: ignore
         # ControlNet
