@@ -1,4 +1,4 @@
 from .vgg import *
-from .register import *
+from .resnet import *
 from .mobilenet import *
 from .transformer import *
