@@ -1,8 +1,9 @@
 # Dependency Hierarchy
-## utils, common, customs, norms
+## utils, common, customs, hooks, norms
+### hijacks (hooks, customs)
 ### activations (common)
 #### mappings (norms, customs, activations)
-#### convs.basic (norms, activations)
+#### convs.basic (norms, hijacks, activations)
 #### convs.residual (convs.basic, utils)
 ##### attentions (convs, common, customs, activations)
 ###### high_level (all the above)
@@ -12,7 +13,9 @@
 from .utils import *
 from .common import *
 from .customs import *
+from .hooks import *
 from .norms import *
+from .hijacks import *
 from .activations import *
 from .mappings import *
 from .convs import *
